#include <config.h>
#define _GNU_SOURCE
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
static const char SPRED_input_variables_definition_arr[] =
"## Definitions of the input variables\n"
"##inputvar: #name of the variable as declared in the code\n"
"## DESCRIPTION: Extended description of the variable to be used for documentation\n"
"## COMMENT: This is the description of the variable as will appear in the logfile\n"
"## RANGE: [from,to] #always put two numbers (also .inf can be put)\n"
"## EXCLUSIVE: #here follows a mapping of allowed values (either RANGE or EXCLUSIVE)\n"
"##  Value1:  comments of value1\n"
"##  Value2:  comment of value2\n"
"##  Alternatively, it follows a list of the allowed values without comments\n"
"## (to be considered in the case of self-explanatory values, e.g. list of integers)\n"
"##  [ Val1, Val2, Val3 ]\n"
"## CONDITION: #here the conditions for which the variable makes sense are written\n"
"##   MASTER_KEY: foo #this means that inputvar makes sense only if foo is specified\n"
"##   WHEN: #provide a list of allowed values of foo for which inputvar is meaningful\n"
"##     - fooval1\n"
"##     - fooval2\n"
"##   WHEN_NOT: #provide a list of forbidden values\n"
"##     - fooval3\n"
"##     - fooval4\n"
"## PROFILE_FROM: foo #inputvar inherit the profile from the one specified by foo variable\n"
"###then the profiles follows, which gives to the variables the allowed name\n"
"## default: value of the default, written as a string \n"
"## profile1: value1 # if the user specifies inputvar: profile1 then inputvars will be value1\n"
"## profile2: value2\n"
"##\n"
"## DO NOT USE the character ':' and "" in the COMMENT of DESCRIPTION\n"
"############################################\n"
" fingerprints:\n"
"   DESCRIPTION: Parameters for structure predicition and reaction pathway search modules\n"
"   fpmethod:\n"
"     COMMENT: fingerprint method\n"
"     EXCLUSIVE:\n"
"       OMF: (O)verlap (M)atrix (F)ree boundary conditions\n"
"       OMP: (O)verlap (M)atrix (P)eriodic boundary conditions\n"
"       OMPOLD: (O)verlap (M)atrix (P)eriodic boundary conditions (old implementation)\n"
"       OMSOLD: (O)verlap (M)atrix (S)lab boundary conditions (old implementation)\n"
"     default: OMF\n"
"   natx_sphere:\n"
"     COMMENT: Number of atoms in the sphere. Only for OMP.\n"
"     default: 250\n"
"     CONDITION:\n"
"       MASTER_KEY: fpmethod\n"
"       WHEN:\n"
"       - OMP\n"
"   angmom:\n"
"     COMMENT: Angular momentum of gaussian orbitals. Only for OMF and OMP.\n"
"     default: 1\n"
"     EXCLUSIVE:\n"
"       1: s-orbitals\n"
"       4: s- and p-orbitals\n"
"     #shortcuts\n"
"     S: 1\n"
"     SP: 4\n"
"     CONDITION:\n"
"       MASTER_KEY: fpmethod\n"
"       WHEN:\n"
"       - OMF\n"
"       - OMP\n"
"       - OMPOLD\n"
"       - OMSOLD\n"
" global:\n"
"   DESCRIPTION: Parameters for the global code (Minima Hopping)\n"
"   nwrite:\n"
"     COMMENT: rewriting frequency of poslows, enarr, idarr, ... \n"
"     default: 1\n"
"   mdmin:\n"
"     COMMENT: stop MD after mdmin minima and maxima along the MD trajectory\n"
"     default: 2\n"
  ;
void FC_FUNC_(get_spred_input_variables_definition, GET_SPRED_INPUT_VARIABLES_DEFINITION)(char* db_ptr,int* db_len)
{
  if (*db_len==0)
    {
    *db_len=strlen(SPRED_input_variables_definition_arr);
    return;
    }
  else
    {
      memcpy(db_ptr,SPRED_input_variables_definition_arr, sizeof(char) * (*db_len));
    }
}
